import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple, Form, LoadingService, Modal, ModalService, Select, Text, Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, MemoryDatasource, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { ITekGridColumn, TekGrid } from '@zeedhi/teknisa-components-common';

export class DashboardTarefasController {
    private modal!: Modal;

    private modalConflito!: Modal;

    public currentRow: any;

    public visibilidadeGrid: boolean = true;

    public async criarModais() {
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00000_000_dash_tarefas/modal/fm_modal_transferir_tarefas' });
        this.modalConflito = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00000_000_dash_tarefas/modal/fm_modal_conflito_transferencia' });
    }

    public destruirModais() {
        this.modal?.destroy();
        this.modalConflito?.destroy();
    }

    public async abrirModal({ component, row }: any) {
        this.modal.show();
        this.visibilidadeGrid = (component.name === 'action-transferir');
        setTimeout(() => {
            this.validacoes(row);
        }, 500);
    }

    public async abrirModalConflito(parametros: any) {
        this.modalConflito.show();
        setTimeout(() => {
            this.preencherGridConflitos(parametros);
         }, 500);
    }

    public fecharModal() {
        this.modal.hide();
    }

    public fecharModalConflito() {
        this.modalConflito.hide();
    }

    public validacoes(row: any) {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino');
        const operadorOrigem = Metadata.getInstance<Select>('operador-origem');

        operadorDestino.value = null;
        operadorOrigem.value = null;

        if (!this.visibilidadeGrid) {
            operadorOrigem.disabled = true;
            operadorDestino.disabled = false;
            operadorOrigem.value = row.CDOPERADOR;
            operadorDestino.datasource.setFilter({
                NRHABILIDADE: row.NRHABILIDADE,
                CDOPERADOR: row.CDOPERADOR,
            });
            this.currentRow = row;
        } else {
            operadorOrigem.disabled = false;
            operadorDestino.disabled = false;
            this.defineTarefa();
        }
    }

    public defineTarefa() {
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        gridInterno.datasource.data = [];
        (gridInterno.datasource as MemoryDatasource).allData = [];

        gridPrincipal.selectedRows.forEach((element: any) => {
            const dados: any = {
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                CDOPERADOR: element.CDOPERADOR,
                DTHORAAGEND: element.DTHORAAGEND,
                NRHABILIDADE: element.NRHABILIDADE,
                DTHRFINALAGEND: element.DTHRFINALAGEND,
                NRPRIORIDADETAREFA: element.NRPRIORIDADETAREFA,
                IDPRIORIDADE: element.IDPRIORIDADE,
            };
            gridInterno.datasource.post(dados);
        });
        gridInterno.datasource.get();
    }

    public disableSelection({ row }: IDictionary) {
        const select = Metadata.getInstance<Select>('operador-origem');
        return row.CDOPERADOR !== select.value;
    }

    public changeOperadorDestino({ component }: IEventParam<Select>) {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino');
        Metadata.getInstance<TekGrid>('grid-tarefas-transferir').selectedRows = [];
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        if (component.value) {
            operadorDestino.datasource.setFilter({
                NRHABILIDADE: component.selectValue.NRHABILIDADE,
                CDOPERADOR: component.value,
            });
        }

        if (this.visibilidadeGrid) {
            gridInterno.datasource.setFilter({
                CDOPERADOR: component.value,
            });
        }
    }

    public focusOperadores({ component, row }: any) {
        component.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
            NRHABILIDADE: row.NRHABILIDADE,
        });
    }

    public preencherGridConflitos(parametros: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-conflito-tarefas');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        /* eslint-disable */
        // if (Array.isArray(parametros)) {
        for (const key in parametros) {
            const dados: any = {
                TAREFABANCO: parametros[key].TAREFABANCO,
                NRCHECKLISTAGEND: parametros[key].NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: parametros[key].NMCHECKLISTAGEND,
                NRLOCALCLIENTE: parametros[key].NRLOCALCLIENTE,
                DTHORAAGEND: parametros[key].DTHORAAGEND,
                HORAINICIAL: parametros[key].HORAINICIAL,
                DURACAO: parametros[key].DURACAO,
                CDOPERADOR: parametros[key].CDOPERADOR,
                IGNORARTAREFA: null,
                NRHABILIDADE: parametros[key].NRHABILIDADE,
                IDPRIORIDADE: parametros[key].IDPRIORIDADE,
            };
            grid.datasource.post(dados);

            const coluna: ITekGridColumn | undefined = grid.columns.find((item: any) => { return item.name === 'CDOPERADOR'});
            coluna?.setLookupDataFilter({ NRLOCALCLIENTE: parametros[key].NRLOCALCLIENTE,  NRHABILIDADE: parametros[key].NRHABILIDADE });
        }
        grid.datasource.get();
        // }
        /* eslint-enable */
        // this.dadosBanco = parametros;
    }

    public transferirTarefa() {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino').value;
        const operadorOrigem = Metadata.getInstance<Select>('operador-origem').value;
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        if (Utilities.validateForm(Metadata.getInstance<Form>('form-transferencia'))) {
            if (operadorOrigem !== operadorDestino) {
                if (this.visibilidadeGrid) {
                    if (gridInterno.selectedRows.length > 0) {
                        this.salvarTransferencia(operadorDestino, gridInterno.selectedRows).then(async (response: IDictionary) => {
                            if (response.data !== 'OK') {
                                this.abrirModalConflito(response.data);
                                Utilities.alertMessage('Ainda há conflitos existentes. Resolva-os para prosseguir.', 'error');
                            } else {
                                Utilities.showSuccessMessage();
                                gridPrincipal.datasource.get();
                                gridInterno.deleteRows();
                                gridInterno.datasource.clearFilter();

                                if (gridInterno.datasource.data.length === 0) {
                                    this.fecharModal();
                                }
                            }
                        }).finally(() => {
                            LoadingService.hide();
                        });
                    } else {
                        Utilities.alertMessage('Selecione pelo menos 1 tarefa para transferir', 'warning');
                    }
                }

                if (!this.visibilidadeGrid) {
                    this.salvarTransferencia(operadorDestino, [this.currentRow]).then((response: IDictionary) => {
                        if (response.data !== 'OK') {
                            this.abrirModalConflito(response.data);
                            Utilities.alertMessage('Ainda há conflitos existentes. Resolva-os para prosseguir.', 'error');
                        } else {
                            Utilities.showSuccessMessage();
                            this.fecharModalConflito();
                            this.fecharModal();
                            gridPrincipal.datasource.get();
                        }
                    }).catch((error: any) => {
                        Utilities.showErrorMessage(error);
                    }).finally(() => {
                        LoadingService.hide();
                    });
                }
            } else {
                Utilities.alertMessage('Selecione um Operador de Destino diferente do Operador de Origem', 'warning');
            }
        }
    }

    public editedRows: any = '';

    public validarGrid({ component }: IEventParam<TekGrid>) {
        if (Utilities.validateGrid(component)) {
            Utilities.showSuccessMessage();
            this.editedRows = component.getEditedRows();
        }
    }

    public salvarConflitos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-conflito-tarefas');
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        if (this.editedRows.length !== 0) {
            this.salvarTransferencia(null, grid.datasource.data).then((response: IDictionary) => {
                if (response.data !== 'OK') {
                    this.abrirModalConflito(response.data);
                    Utilities.alertMessage('Ainda há conflitos existentes. Resolva-os para prosseguir.', 'error');
                } else {
                    Utilities.showSuccessMessage();
                    this.fecharModalConflito();
                    gridInterno.deleteRows();
                    gridInterno.datasource.clearFilter();

                    if (gridInterno.datasource.data.length === 0) {
                        this.fecharModal();
                    }
                 }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Nenhuma alteração indentificada. Corrija os conflitos ou cancele a operação', 'error');
        }
    }

    public async salvarTransferencia(destino: any, tarefa: any) {
        return Http.post('/transferirTarefas', {
            OPERADORDESTINO: destino,
            TAREFAS: tarefa,
        });
    }

    public setFiltro({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
        this.changeMaxWidth({ component });
    }

    public montarInformacoesComplementares() {
        LoadingService.show();
        const nr = this.currentRow.NRCHECKLISTAGEND;
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-itens');
        const observacoes = Metadata.getInstance<Textarea>('observacoes-tarefas');
        const semItens = Metadata.getInstance<Text>('nao-ha-itens');
        let mensagem = '';

        Http.post('/getItensTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: IDictionary) => {
            const dados = response.data.data;

            checkbox.value = dados.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA);
            (checkbox.datasource as SimpleDatasource)!.items = dados.map((elemento: any) => elemento.NMTAREFA);

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            if (this.currentRow.DSJUSTCANCELAMENTO !== null && this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO} - Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else if (this.currentRow.DSJUSTCANCELAMENTO !== null) {
                mensagem = `Justificativa de cancelamento: ${this.currentRow.DSJUSTCANCELAMENTO}`;
            } else if (this.currentRow.DSOBSTAREFAAGEND !== null) {
                mensagem = `Observações da Tarefa: ${this.currentRow.DSOBSTAREFAAGEND}`;
            } else {
                mensagem = 'Não há observações';
            }
            observacoes.value = mensagem;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarFotos({ component }: any) {
        const nr = this.currentRow.NRCHECKLISTAGEND;

        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: nr,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public disabledAction: boolean = false;

    public rowStyleConditions(row: any) {
        if (row.ATRASO !== null && row.ATRASO.split(' ')[1] === 'dias' && row.ATRASO.split(' ')[0] >= 1) {
            return { cssClass: 'linha-grid-verificada' };
        }

        if (row.IDSITUACAOAGEND === '4') {
            return { cssClass: 'linha-grid-concluida' };
        }

        return { cssClass: '' };
    }

    private clicks: any = 0;

    // função que simula um double click
    public rowClick({ row }: any) {
        this.currentRow = row;
        this.clicks += 1;
        if (this.clicks >= 2) {
            // this.abrirModal();
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        }, 500);
    }

    public canEditRow({ row }: IEventParam<TekGrid>) {
        return (row.TAREFABANCO !== 'N');
    }

    public disabledSelectionGrid({ row }: any) {
        return (row.IDSITUACAOAGEND === '4' || row.IDSITUACAOAGEND === '3');
    }

    public visibleActionGrid({ row }: any) {
        return !(row.IDSITUACAOAGEND === '4' || row.IDSITUACAOAGEND === '3');
    }

    public changeMaxWidth({ component }: IEventParam<TekGrid>) {
        /* eslint-disable */
        const coluna: ITekGridColumn | undefined = component.columns.find((item: any) => { return item.name === 'NMTAREFAS' });
        /* eslint-enable */

        if (coluna) {
            coluna.maxWidth = '250px';
            coluna.overflow = 'wrap';
        }
    }
}
