/* eslint-disable */
import { Utilities } from '@/util/Utilities';
import {
    Button,
    DialogService,
    Form, LoadingService, Modal, ModalService, Select, SelectMultiple, TextInput, TreeGrid,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { TekGrid } from '@zeedhi/teknisa-components-vuetify';

export class Fm00003000ChecklistController {
    public modalChecklist!: Modal;

    public edicaoChecklist!: Modal;

    public currentRow: any;

    public dados: any;

    public dadosChecklist: any = {};

    public resposta: any;

    public validaEdicao: any;

    public nrEdicao: any;

    public visibilidadeGrid: boolean = false;

    public visibilidadeBotaoGerar: boolean = !this.visibilidadeGrid;

    public check: boolean = false;

    public async criarModais() {
        this.modalChecklist = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00003_000_checklist/modal/fm_00003_001_salvar_checklist' });
        this.edicaoChecklist = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00003_000_checklist/modal/fm_00003_004_editar_checklist' });
    }

    public destruirModais() {
        this.modalChecklist?.destroy();
        this.edicaoChecklist?.destroy();
    }

    public async abrirModal() {
        this.modalChecklist.show();
    }

    public fecharModal() {
        this.modalChecklist.hide();
    }

    public gerarDadosChecklist() {
        const form = Metadata.getInstance<Form>('form-checklist');
        const treeGrid = Metadata.getInstance<TekTreeGrid>('tree-grid-checklist');
        const selectAtividades = Metadata.getInstance<SelectMultiple>('atividades-checklist');

        if (Utilities.validateForm(form)) {
            this.visibilidadeGrid = true;
            treeGrid.datasource.setFilter({
                NRTAREFA: selectAtividades.value,
            });
        }
    }

    public changeAtividades(atividades: any) {
        const selectAtividades = Metadata.getInstance<Select>('atividades-checklist');
        if (atividades.length > 0) {
            selectAtividades.disabled = false;
            selectAtividades.datasource.setFilter({
                NRATIVIDADE: atividades,
            });
        } else {
            selectAtividades.disabled = true;
            selectAtividades.value = '';
            this.visibilidadeGrid = false;
        }
    }

    public changeVisibilidadeGrid({ component }: any) {
        if (component.value.length > 0) {
            this.visibilidadeGrid = false;
        }
        this.reloadGridEdicao(component.value);
    }

    public habilitarBotaoSalvar() {
        Metadata.getInstance<Button>('salvar-checklist').disabled = true;
    }

    public async salvarChecklist() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-checklist');
        const tekGrid = Metadata.getInstance<TekTreeGrid>('tek-grid-checklist');

        if (grid.selectedRows.length > 0) {
            const escopos = (grid.selectedRows.filter((elemento: any) => elemento.TIPO === 'Atividade')).map((item: any) => ({ NRTAREFA: item.CODIGOAUX, NRESCOPO: item.CODIGOPAIAUX }));
            const criterios = (grid.selectedRows.filter((elemento: any) => elemento.TIPO === 'Critério')).map((item: any) => ({ NRCRITERIO: item.CODIGOAUX, NRTAREFA: item.CODIGOPAIAUX }));

            await Utilities.save('/salvarChecklist', {
                ESCOPOS: escopos,
                CRITERIOS: criterios,
                NMCHECKLIST: Metadata.getInstance<TextInput>('nome-checklist').value,
                NRTIPOLOCALCLI: Metadata.getInstance<Select>('tipo-local-checklist').value,
                NRATIVIDADE: Metadata.getInstance<SelectMultiple>('itens-checklist').value,
                NRTAREFA: Metadata.getInstance<SelectMultiple>('atividades-checklist').value,
            }).then((Response) => {
                this.resposta = Response;
                this.fecharModal();
                this.fecharModalEdicao();
                tekGrid.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        } else {
            Utilities.alertMessage('Algum escopo deve ser selecionado', 'error');
            this.resposta = "error";
        }
    }

    // EDIÇÃO MODAL CHECKLIST
    public async abrirModalEdicao({ row }: any) {
        this.edicaoChecklist.show();
        this.getTipoLocalModalEdicao(row);
        this.currentRow = row;
    }

    public fecharModalEdicao() {
        this.edicaoChecklist.hide();
        this.dados = [];
        this.currentRow = [];
    }

    public getTipoLocalModalEdicao(row: any) {
        LoadingService.show();
        Http.post('/getTipoLocalModalEdicao', {
            NMTIPOLOCALCLI: row.NMTIPOLOCALCLI,
        }).then((Response) => {
            Metadata.getInstance<TextInput>('nome-checklist').value = row.NMCHECKLIST;
            Metadata.getInstance<Select>('tipo-local-checklist').setValue(Response.data.data[0].NRTIPOLOCALCLI);
            this.dadosChecklist.NRTIPOLOCALCLI = Response.data.data[0].NRTIPOLOCALCLI;
            this.dadosChecklist.NRCHECKLIST = this.currentRow.NRCHECKLIST;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            this.getItensModalEdicao(row);
            LoadingService.hide();
        });

    }

    public getItensModalEdicao(row: any) {
        LoadingService.show();
        Http.post('/getItensModalEdicao', {
            NRCHECKLIST: row.NRCHECKLIST,
        }).then((Response) => {
            this.dados = Response.data.data[0].NRATIVIDADE;
            Metadata.getInstance<SelectMultiple>('itens-checklist').setValue([Response.data.data.map((item: any) => item.NRATIVIDADE)][0]);
            this.dadosChecklist.NRATIVIDADE = [Response.data.data.map((item: any) => item.NRATIVIDADE)][0];
            this.validaChanges(this.dados);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            this.getIAtividadesModalEdicao(row);
            LoadingService.hide();
        });
    }

    public reloadGridEdicao(atividadeEdicao: any) {
        const treeGrid = Metadata.getInstance<TreeGrid>('tree-grid-checklist');
        treeGrid.datasource.data = [];
        treeGrid.datasource.setFilter({
            NRTAREFA: atividadeEdicao,
        });
    }

    public getIAtividadesModalEdicao(row: any) {
        Http.post('/getAtividadesModalEdicao', {
            NRCHECKLIST: row.NRCHECKLIST,
        }).then((Response) => {
            setTimeout(() => {
                Metadata.getInstance<SelectMultiple>('atividades-checklist').setValue(Response.data[0].RETORNO.data.map((item: any) => item.NRTAREFA));
                const treeGrid = Metadata.getInstance<TreeGrid>('tree-grid-checklist');
                treeGrid.datasource.setFilter({
                    NRTAREFA: Response.data[0].RETORNO.data.map((item: any) => item.NRTAREFA),
                });
                this.desabilitaEdicao();
            }, 2000);
            this.dadosChecklist.NRTAREFA = Response.data[0].RETORNO.data.map((item: any) => item.NRTAREFA);
            this.validaEdicao = Response.data[0].NRCHECKLIST;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public validaChanges({ component }: any) {
        const selectAtividades = Metadata.getInstance<Select>('atividades-checklist');
        if (!component) {
            selectAtividades.disabled = false;
            selectAtividades.datasource.setFilter({
                NRATIVIDADE: [this.dados],
            });
        } else {
            this.changeAtividades(component.value);
        }
    }

    public desabilitaEdicao() {
        this.validaEdicao.forEach((element: any) => {
            if (this.currentRow.NRCHECKLIST === element) {
                Metadata.getInstance<Select>('tipo-local-checklist').disabled = true;
                Metadata.getInstance<SelectMultiple>('itens-checklist').disabled = true;
                Metadata.getInstance<SelectMultiple>('atividades-checklist').disabled = true;
                Metadata.getInstance<TreeGrid>('tree-grid-checklist').selectable = false;
                this.dadosChecklist.NOME = 'S';
            }
        });
    }

    public async editarChecklist() {
        const tekGrid = Metadata.getInstance<TekTreeGrid>('tek-grid-checklist');
        this.dadosChecklist.NMCHECKLIST = Metadata.getInstance<TextInput>('nome-checklist').value;
        LoadingService.show();
        if (this.dadosChecklist.NOME !== 'S') {
            await this.salvarChecklist();
        }
        if (this.dadosChecklist.NOME === 'S' || this.resposta != 'error' || this.resposta.data === 'ok') {
            Http.post('/edicaoChecklist', {
                CHECKLISTATUAL: this.dadosChecklist,
            }).then(() => {
                this.fecharModalEdicao();
                tekGrid.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.dadosChecklist = {};
            });
        }
        LoadingService.hide();
    }

    public dialogInativarChecklist({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00003000ChecklistController.inativarChecklist', 'Inativar Registro', 'Deseja realmente inativar todos os registros selecionados?');
    }

    public inativarChecklist() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-checklist');

        Utilities.deleteInfoGrid('/inativarChecklist', { component: grid }).then((response: any) => {
            DialogService.hide();
            if (response.data === 'erro') {
                Utilities.showDialog('Atenção!', 'warning', 'Alguns registros não puderam ser inativados pois estão sendo utilizados!');
            }
        })
    }
}
