import { Utilities } from '@/util/Utilities';
import {
    Button, Date as Data, DateRange, DialogService, Form, LoadingService, Modal, ModalService, Select, SelectMultiple, Tabs, TextInput,
} from '@zeedhi/common';
import {
    Http, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00040000AuditoriaController {
    public modalTarefas!: Modal;

    public modalModelo!: Modal;

    public modalPreencherInformacoes!: Modal;

    private parametrizacao: any;

    public async criarModais() {
        this.modalTarefas = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00040_000_auditoria/modal/fm_00040_005_cadastro_tarefas_auditoria' });
        this.modalModelo = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00040_000_auditoria/modal/fm_00040_003_adicionar_modelo_auditoria' });
        this.modalPreencherInformacoes = await ModalService.createFromJson('fm_00040_000_auditoria/modal/fm_00040_005_modal_preencher_massa', true);
    }

    public destruirModais() {
        this.modalTarefas?.destroy();
        this.modalModelo?.destroy();
        this.modalPreencherInformacoes?.destroy();
    }

    public abrirModal() {
        this.modalTarefas.show();
        /* setTimeout(() => {
            Metadata.getInstance<Form>('form-cadastro-auditoria').clearForm();
            Metadata.getInstance<Form>('formTarefas_dropBoxDadosAgendamento').clearForm();
        }, 500); */
    }

    public fecharModal() {
        this.modalTarefas.hide();
    }

    public abrirModalInserirModelo() {
        this.modalModelo.show();
    }

    public fecharModalInserirModelo() {
        this.modalModelo.hide();
    }

    public visibilidade({ component }: IEventParam<Tabs>) {
        const botao = Metadata.getInstance<Button>('botao-acionar-adicao');

        if (component.tabs[component.activeTab].name === 'tab-modelo-auditoria') {
            botao.isVisible = true;
        } else {
            botao.isVisible = false;
        }
    }

    public clickButtonToExpand({ component }: any) {
        const form = Metadata.getInstance<Form>(`formTarefas_${component.name}`);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public changeTipoDeLocal({ component }: IEventParam<Select>) {
        const tipoLocal = Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI');
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');

        if (component.value) {
            tipoLocal.disabled = false;
            tipoLocal.datasource.setFilter({
                CDCLIENTE: component.value,
            });
        } else {
            tipoLocal.disabled = true;
            tipoLocal.value = null;
            local.value = null;
            local.disabled = true;
        }
    }

    public changeLocal({ component }: IEventParam<SelectMultiple>) {
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');

        if (component.value > 0) {
            local.disabled = false;
            local.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'N',
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
            });
        } else {
            local.value = null;
            local.disabled = true;
        }
    }

    public changeContratos({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento-auditoria');
        if (grid.editing) {
            Metadata.getInstance<Select>(component.name).datasource.setFilter({
                NRESTRUTURAM: grid.datasource.currentRow.NRESTRUTURAM,
            });
            grid.columns[1].setLookupDataFilter({ NRESTRUTURAM: grid.datasource.currentRow.NRESTRUTURAM });
        }
    }

    public focusOperadores({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento-auditoria');
        if (grid.editing) {
            component.datasource.setFilter({
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
            });
        }
    }

    public montarGridAuditorias({ component }: IEventParam<SelectMultiple>) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento-auditoria');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        // const locais = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').selectValue;
        component.selectValue.forEach((element: any) => {
            const dados: any = {
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NRESTRUTURAM: element.NRESTRUTURAM,
                NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
        grid.columns[3].setLookupDataFilter({ NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value });
    }

    disabledPreencher: boolean = true;

    public habilitarAcao({ component }: IEventParam<TekGrid>) {
        this.disabledPreencher = (component.selectedRows.length <= 0);
    }

    public abrirModalPreencher() {
        this.modalPreencherInformacoes.show();

        setTimeout(() => {
            Metadata.getInstance<Form>('form-preencher-massa').clearForm();
            //  trecho do modal de preenchimento em massa
            Metadata.getInstance<Select>('select-operador').datasource.setFilter({
                NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
            });
        }, 500);
    }

    public fecharModalPreencher() {
        this.modalPreencherInformacoes.hide();
    }

    public preencherGrid() {
        const form = Metadata.getInstance<Form>('form-preencher-massa');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento-auditoria');
        const operador = Metadata.getInstance<Select>('select-operador').value;
        const opcoes = Metadata.getInstance<Select>('selectOpcoesMarcacao').value;

        if (Utilities.validateForm(form)) {
            grid.selectedRows.forEach((element: any) => {
                const newRow: any = {
                    ...element,
                    OPERADOR: operador,
                    IDTIPOMARCACAO: opcoes,
                };
                grid.datasource.put(newRow);
            });
            grid.datasource.get();
            grid.selectedRows = [];
            this.fecharModalPreencher();
        }
    }

    public changeRecorrencia({ component }: IEventParam<Select>) {
        const data = Metadata.getInstance<Data>('select-data');
        const xDias = Metadata.getInstance<TextInput>('input-x-dias');
        const periodo = Metadata.getInstance<DateRange>('select-periodo');
        const dtFinal = Metadata.getInstance<Data>('select-data-limite');

        switch (component.value) {
            case 'SR':
                periodo.disabled = false;
                data.removeValidation('required');
                data.value = null;
                periodo.addValidation('required', {});
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.removeValidation('required');
                dtFinal.disabled = true;
                break;
            case 'M':
                data.addValidation('required', {});
                periodo.disabled = true;
                periodo.removeValidation('required');
                periodo.value = null;
                data.disabled = false;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'MP':
                periodo.addValidation('required', {});
                periodo.disabled = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'Q':
                data.addValidation('required', {});
                periodo.disabled = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'QP':
                periodo.addValidation('required', {});
                periodo.disabled = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'S':
                data.addValidation('required', {});
                periodo.disabled = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'SP':
                periodo.addValidation('required', {});
                periodo.disabled = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'P':
                data.addValidation('required', {});
                periodo.disabled = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = false;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            default:
                break;
        }
    }

    public onMountedTipoInsercao({ component }: IEventParam<Select>) {
        component.datasource.data = [];
        (component.datasource as MemoryDatasource).allData = [];

        const valores = [
            {
                name: 'Considerar todos os dias',
                value: 1,
            },
            {
                name: 'Considerar dias úteis (Criar tarefa no próximo dia útil)',
                value: 2,
            },
            {
                name: 'Considerar dias úteis (Não criar tarefa)',
                value: 3,
            },
        ];

        setTimeout(() => {
            if (this.parametrizacao === 'N') {
                const codARemover = 2;
                const index = valores.findIndex((item: any) => item.value === codARemover);

                if (index !== -1) {
                    valores.splice(index, 1);
                }
            } else {
                valores.splice(1, 0, {
                    name: 'Considerar dias úteis (Criar tarefa no próximo dia útil)',
                    value: 2,
                });
            }

            component.datasource.updateData(valores);
        }, 500);
    }

    public executorHora() {
        const dados: any = [];
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento-auditoria');

        grid.datasource.data.forEach((element: any) => {
            dados.push({
                CDOPERADOR: element.OPERADOR,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NRSEQCONTRATO: element.NRSEQCONTRATO,
                NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
                IDTIPOMARCACAO: element.IDTIPOMARCACAO,
            });
        });
        return dados;
    }

    public inserirTarefa() {
        const formTarefas = Metadata.getInstance<Form>('form-cadastro-auditoria');
        const formAgendamento = Metadata.getInstance<Form>('formTarefas_dropBoxDadosAgendamento');
        const diaSemana = Metadata.getInstance<SelectMultiple>('selectDiaSemana').value;
        const dataRange = Metadata.getInstance<DateRange>('select-periodo').value;
        const data = Metadata.getInstance<DateRange>('select-data').value;

        if (Utilities.validateForm(formTarefas) && Utilities.validateForm(formAgendamento)) {
            const parametros = {
                NMAUDITORIA: Metadata.getInstance<TextInput>('nome-tarefa').value,
                NRMODELOAUDIT: Metadata.getInstance<Select>('NRMODELOAUDIT').value,
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
                NRTIPOLOCALCLI: Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI').value,
                NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
                DATERANGE: (dataRange.length === 0) ? [data, data] : dataRange,
                DIASEMANA: diaSemana,
                OBSERVACOES: Metadata.getInstance<Select>('selectObservacoesObrigatorias').value,
                EXECUTOR: this.executorHora(),
                RECORRENCIA: Metadata.getInstance<Select>('select-recorrencia').value,
                QTDDIAS: Metadata.getInstance<TextInput>('input-x-dias').value,
                DTTERMINO: Metadata.getInstance<Data>('select-data-limite').value,
            };
            this.salvarTarefa(parametros, this.executorHora());
        }
    }

    public salvarTarefa(parametros: any, executor: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-tarefas-auditoria');

        LoadingService.show();
        Http.post('/inserirTarefaAuditoria', {
            NMAUDITORIA: parametros.NMAUDITORIA,
            NRMODELOAUDIT: parametros.NRMODELOAUDIT,
            CDCLIENTE: parametros.CDCLIENTE,
            NRTIPOLOCALCLI: parametros.NRTIPOLOCALCLI,
            NRLOCALCLIENTE: parametros.NRLOCALCLIENTE,
            DATERANGE: parametros.DATERANGE,
            DIASEMANA: parametros.DIASEMANA,
            OBSERVACOES: parametros.OBSERVACOES,
            EXECUTOR: executor,
            RECORRENCIA: parametros.RECORRENCIA,
            QTDDIAS: parametros.QTDDIAS,
            DTTERMINO: parametros.DTTERMINO,
        }).then(() => {
                Utilities.showSuccessMessage();
                this.fecharModal();
                grid.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public excluirTarefas({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040000AuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-tarefas-auditoria');
        LoadingService.show();
        Http.post('/excluirRegistros', {
            AUDITORIA: component.selectedRows.map((element: any) => element.NRAUDITORIA),
        }).then(() => {
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
